// StickyDialog.cpp : implementation file
//

#include "stdafx.h"
#include "StickyDialog.h"
#include ".\stickydialog.h"

CStickyDragManager CStickyDialog::m_dragManager;

// CStickyDialog dialog
IMPLEMENT_DYNAMIC(CStickyDialog, CDialog)

bool CStickyDialog::IsDocker(void) const
{
	return m_dragManager.GetDocker(this);
}

bool CStickyDialog::IsDockable(void) const
{
	return m_dragManager.GetDockable(this);
}

void CStickyDialog::Docker(const bool bDocker)
{
	m_dragManager.SetDocker(this,bDocker);
}

void CStickyDialog::Dockable(const bool bDockable)
{
	m_dragManager.SetDockable(this,bDockable);
}

CStickyDialog::~CStickyDialog()
{
}

BEGIN_MESSAGE_MAP(CStickyDialog, CDialog)
	ON_WM_CLOSE()
	ON_WM_MOVING()
	ON_WM_SIZING()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
END_MESSAGE_MAP()

// CStickyDialog message handlers

void CStickyDialog::OnClose()
{
	//    .

	Dockable(false);
	m_dragManager.DeleteWindow(this);
	CDialog::OnClose();
}

void CStickyDialog::OnMoving(UINT fwSide, LPRECT pRect)
{
	//   
	if (m_dragManager.GetDockable(this))
	{ //  
		POINT pt;
		GetCursorPos(&pt);
		RECT saveR=*pRect;
		if (EqualRect(pRect,&saveR)) //   
			OffsetRect(pRect,pt.x-saveR.left-m_relativeOffset.x,pt.y-saveR.top-m_relativeOffset.y);
		m_dragManager.TryDock(this,pRect); //    
		if (m_dragManager.GetDocker(this)) //    
			m_dragManager.MoveDocked(this,pRect); //   
		//  
		SetWindowPos(0,pRect->left,pRect->top,0,0,SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER);
	}
	else
		CDialog::OnMoving(fwSide,pRect);
}

void CStickyDialog::OnSizing(UINT fwSide, LPRECT pRect)
{
	//       .
	m_dragManager.Resize(this,pRect);
}

void CStickyDialog::OnNcLButtonDown(UINT nHitTest, CPoint point)
{
	if (nHitTest==HTCAPTION)
	{	//        
		//         .
		//       -   
		RECT r;
		GetWindowRect(&r);
		m_relativeOffset.x=point.x-r.left;
		m_relativeOffset.y=point.y-r.top;
		m_dragManager.FixDockState(this);
	}
	CDialog::OnNcLButtonDown(nHitTest, point);
}

BOOL CStickyDialog::OnInitDialog()
{
	//    .

	CDialog::OnInitDialog();
	m_dragManager.AddWindow(this);
	Dockable(true);
	return TRUE;
}

void CStickyDialog::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow,nStatus);
	//      ""
	Dockable(bShow==TRUE);
}
