#include "stdafx.h"
#include "StickyDragManager.h"


CStickyDragManager::CStickyDragManager(void)
{
	// 

	m_slack=5;
	InitializeCriticalSection(&m_lock);
}

CStickyDragManager::~CStickyDragManager()
{
	DeleteCriticalSection(&m_lock);
}

void CStickyDragManager::AddWindow(const CWnd *pWnd)
{
	//    
	if (IsWindow(pWnd->m_hWnd))
	{
		EnterCriticalSection(&m_lock);
		m_windows.insert(MapType((CWnd *)pWnd,CWindowInfo(pWnd)));
		LeaveCriticalSection(&m_lock);
	}
}

void CStickyDragManager::DeleteWindow(const CWnd *pWnd)
{
	//   
	EnterCriticalSection(&m_lock);
	m_windows.erase((CWnd *)pWnd);
	LeaveCriticalSection(&m_lock);
}

void CStickyDragManager::UndockAll(WndList *list)
{
	if (list)
		list->clear();
	for (MapIter iter=m_windows.begin();iter!=m_windows.end();iter++)
	{
		if (iter->second.Dockable())
		{
			iter->second.Docker(NULL);
			if (list)
				list->push_back(&(iter->second));
		}
	}
}

void CStickyDragManager::GetDockedWindows(const CWnd *pWnd,WndList &list) const
{
	//    ,    ( )
	list.clear();
	WndList dlist;
	((CStickyDragManager *)this)->UndockAll(&dlist);
	GetDockedWindowsEx(pWnd,list,dlist);
	((CStickyDragManager *)this)->UndockAll();
}

void CStickyDragManager::GetDockedWindowsEx(const CWnd *pWnd,WndList &ldocked,WndList &lnondocked) const
{
	//         ,   
	
	MapCIter fiter=m_windows.find((CWnd *)pWnd);
	if (fiter!=m_windows.end())
	{
		CWindowInfo mwnd=fiter->second;
		for (WndList::iterator iter=lnondocked.begin();iter!=lnondocked.end();iter++)
		{
			if (((CWnd *)(**iter))!=pWnd && mwnd.CheckStick(**iter,0) && (*iter)->Docker()==NULL)
			{
				ldocked.push_back(*iter);
				(*iter)->Docker(pWnd);
				GetDockedWindowsEx(((CWnd *)(**iter)),ldocked,lnondocked);
			}
		}
	}
}

void CStickyDragManager::SetDocker(const CWnd *pWnd,const bool bDocker)
{
	if (IsWindow(pWnd->m_hWnd))
	{
		EnterCriticalSection(&m_lock);
		MapIter fiter=m_windows.find((CWnd *)pWnd);
		if (fiter!=m_windows.end())
			fiter->second.SetDocker(bDocker);
		LeaveCriticalSection(&m_lock);
	}
	else
		DeleteWindow(pWnd);
}

void CStickyDragManager::SetDockable(const CWnd *pWnd,const bool bDockable)
{
	if (IsWindow(pWnd->m_hWnd))
	{
		EnterCriticalSection(&m_lock);
		MapIter fiter=m_windows.find((CWnd *)pWnd);
		if (fiter!=m_windows.end())
			fiter->second.Dockable(bDockable);
		LeaveCriticalSection(&m_lock);
	}
	else
		DeleteWindow(pWnd);
}

bool CStickyDragManager::GetDocker(const CWnd *pWnd) const
{
	bool ret=false;
	if (IsWindow(pWnd->m_hWnd))
	{
		EnterCriticalSection(&m_lock);
		MapCIter fiter=m_windows.find((CWnd *)pWnd);
		if (fiter!=m_windows.end())
			ret=fiter->second.IsDocker();
		LeaveCriticalSection(&m_lock);
	}
	return ret;
}

bool CStickyDragManager::GetDockable(const CWnd *pWnd) const
{
	bool ret=false;
	if (IsWindow(pWnd->m_hWnd))
	{
		EnterCriticalSection(&m_lock);
		MapCIter fiter=m_windows.find((CWnd *)pWnd);
		if (fiter!=m_windows.end())
			ret=fiter->second.Dockable();
		LeaveCriticalSection(&m_lock);
	}
	return ret;
}

void CStickyDragManager::GetAttachedWindows(const CWnd *pWnd,WndList &list) const
{
	//  ,   ,   

	list.clear();
	for (MapCIter iter=m_windows.begin();iter!=m_windows.end();iter++)
	{
		CWnd *dd=iter->second.Docker();
		if (iter->second.Dockable() && iter->second.Docker()==pWnd)
			list.push_back((CWindowInfo*)&(iter->second));
	}
}

void CStickyDragManager::MoveDocked(const CWnd *pWnd,LPRECT pRect)
{
	//     

	EnterCriticalSection(&m_lock);
	WndList wndlist;
	GetAttachedWindows(pWnd,wndlist);
	HDWP hDWP = BeginDeferWindowPos(wndlist.size());
	for (WndList::iterator iter=wndlist.begin();iter!=wndlist.end();iter++)
	{
		CPoint deltas;
		CRect oldrect;
		pWnd->GetWindowRect(oldrect);
		deltas.x=oldrect.left-pRect->left;
		deltas.y=oldrect.top-pRect->top;
		CRect newr=(*iter)->Move(deltas);
		::DeferWindowPos(hDWP,HWND(**iter),0,newr.left,newr.top,0,0,SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER);	
	}
	EndDeferWindowPos(hDWP);
	LeaveCriticalSection(&m_lock);
}

void CStickyDragManager::TryDock(const CWnd *pWnd,LPRECT pRect)
{
	//     -  ,
	//  ,  

	EnterCriticalSection(&m_lock);
	CPoint deltas;
	MapIter fiter=m_windows.find((CWnd *)pWnd);
	if (fiter!=m_windows.end())
	{
		for (MapIter iter=m_windows.begin();iter!=m_windows.end();iter++)
		{
			if (iter->first!=pWnd && iter->second.Dockable())
			{
				if (iter->second.NeedDock(pRect,deltas,m_slack) && (iter->second.Docker()!=pWnd))
					OffsetRect(pRect,deltas.x,deltas.y);
			}
		}
	}
	LeaveCriticalSection(&m_lock);
}

void CStickyDragManager::FixDockState(const CWnd *pWnd)
{
	//          
	// 

	EnterCriticalSection(&m_lock);
	MapIter fiter=m_windows.find((CWnd *)pWnd);
	if (fiter!=m_windows.end())
	{
		if (fiter->second.IsDocker())
		{
			CWindowInfo mwnd=fiter->second;
			WndList wndlist;
			GetDockedWindows(pWnd,wndlist);
			for (WndList::iterator iter=wndlist.begin();iter!=wndlist.end();iter++)
			{
				if (pWnd!=(CWnd *)(**iter))
					(*iter)->Docker((CWnd *)fiter->second);
				else
					(*iter)->Docker(NULL);
			}
		}
	}
	LeaveCriticalSection(&m_lock);
}

void CStickyDragManager::Resize(const CWnd *pWnd,LPRECT r)
{
	RECT cr, dr, sr;
	EnterCriticalSection(&m_lock);
	for (MapIter iter=m_windows.begin();iter!=m_windows.end();iter++)
	{
		if (pWnd==iter->first)
			continue;
		iter->first->GetWindowRect(&cr);
		sr = cr;
		InflateRect(&sr, m_slack, m_slack);
		if (!IntersectRect(&dr, &sr, r))
			continue;
		if (iter->second.IsClose(r->right,cr.left,m_slack))
			r->right-=r->right-cr.left;
		if (iter->second.IsClose(r->left,cr.right,m_slack))
			r->left-=r->left-cr.right;
		if (iter->second.IsClose(r->top,cr.bottom,m_slack))
			r->top-=r->top-cr.bottom;
		if (iter->second.IsClose(r->bottom,cr.top,m_slack))
			r->bottom-=r->bottom-cr.top;
	}
	LeaveCriticalSection(&m_lock);
}