#include "stdafx.h"
#include ".\windowinfo.h"

CWindowInfo::CWindowInfo(const CWnd *pWnd)
{
	m_docker=NULL;
	m_isdocker=false;
	m_main=(CWnd *)pWnd;
	if (m_main && IsWindow(m_main->GetSafeHwnd()))
		m_docable=(m_main->IsWindowVisible()==TRUE);
}

CWindowInfo::~CWindowInfo(void)
{
}

bool CWindowInfo::IsDocker(void) const
{
	return m_isdocker;
}

void CWindowInfo::SetDocker(const bool bDocker)
{
	m_isdocker=bDocker;
}

CWindowInfo::operator CWnd *(void) const
{
	return m_main;
}

CWindowInfo::CWindowInfo(const CWindowInfo& copy)
{
	*this=copy;
}

CWindowInfo &CWindowInfo::operator =(const CWindowInfo& copy)
{
	if (this!=&copy)
	{
		m_docable=copy.m_docable;
		m_docker=copy.m_docker;
		m_isdocker=copy.m_isdocker;
		m_main=copy.m_main;
	}
	return *this;
}

bool CWindowInfo::Dockable(void) const
{
	return m_docable;
}

void CWindowInfo::Dockable(const bool bDockable)
{
	m_docable=bDockable;
}

CWnd *CWindowInfo::Docker(void) const
{
	return m_docker;
}

CWindowInfo::operator HWND(void) const
{
	return m_main->GetSafeHwnd();
}

void CWindowInfo::Docker(const CWnd *pWnd)
{
	m_docker=(CWnd *)pWnd;
}

CRect CWindowInfo::Move(const CPoint &pp)
{
	CRect oldrect;
	m_main->GetWindowRect(oldrect);
	OffsetRect(oldrect,-pp.x,-pp.y);
	return oldrect;
}

bool CWindowInfo::CheckStick(const CWindowInfo& cwi,const int slack)
{
	CRect frect,srect;
	m_main->GetWindowRect(frect);
	((CWnd *)cwi)->GetWindowRect(srect); //
	return (Intersect(frect,srect,1) && (IsStick(frect.left,srect.right) || IsStick(frect.right,srect.left)
		|| IsStick(frect.top,srect.bottom) || IsStick(frect.bottom,srect.top)));
}

bool CWindowInfo::Intersect(const CRect &r1,const CRect &r2,const int slack)
{
	CRect intrect;
	CRect rr1=r1;
	rr1.InflateRect(slack,slack);
	return (intrect.IntersectRect(rr1,r2)!=0);
}

bool CWindowInfo::NeedDock(LPRECT mrect,CPoint &delta,const int slack)
{
	CRect rect;
	m_main->GetWindowRect(rect);
	if (!Intersect(mrect,rect,slack))
		return false;
	delta.SetPoint(0,0);
	if (IsClose(rect.left,mrect->right,slack))
	{
		delta.x=rect.left-mrect->right;
	}
	else if (IsClose(rect.right,mrect->left,slack))
	{
		delta.x=rect.right-mrect->left;
	}
	else if (IsClose(rect.top,mrect->bottom,slack))
	{
		delta.y=rect.top-mrect->bottom;
	}
	else if (IsClose(rect.bottom,mrect->top,slack))
	{
		delta.y=rect.bottom-mrect->top;
	}
	return (delta.x!=0 || delta.y!=0);
}
